void loadData() {
    // load top checkout titles
    for (int i = START_YEAR; i <= END_YEAR; i++) {
        String filename = "top50_" + String.valueOf(i) + ".csv";
        table = loadTable(filename, "header");
        rows = table.getRowCount();
        List<String> titles = new ArrayList();
        for (int j = 0; j < rows; j++) {
            String title = table.getString(j, 1);
            titles.add(title);
        }
        titleMap.put(String.valueOf(i), titles);
    }
    loadAllTime();
    // load checkout record from csv files
    for (int i = START_YEAR; i <= END_YEAR; i++) {
        String filename = "cout_" + String.valueOf(i) + ".csv";
        table = loadTable(filename, "header");
        rows = table.getRowCount();
        List<Record> records = new ArrayList();
        for (int j = 0; j < rows; j++) {
            String title = table.getString(j, 0);
            int itemNumber = table.getInt(j, 1);
            LocalDateTime cout = convertToDate(table.getString(j, 2));
            LocalDateTime cin = convertToDate(table.getString(j, 3));
            String itemType = table.getString(j, 4);
            String deweyClass = table.getString(j, 5);
            Record record = new Record(title, itemNumber, cout, cin, itemType, deweyClass);
            records.add(record);
        }
        recordMap.put(i, records);
    }
    println("Finished loading records");
}

void loadAllTime() {
    String filename = "top1000_alltime.csv";
    table = loadTable(filename, "header");
    rows = table.getRowCount();
    List<String> titles = new ArrayList();
    for (int i = 0; i < 50; i++) {
        String title = table.getString(i, 1);
        titles.add(title);
    }
    titleMap.put("AllTime", titles);
}

LocalDateTime convertToDate(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
    LocalDateTime date = LocalDateTime.parse(str, formatter);
    return date;
}

// return color with semi transparency
color getSemi(color c) {
    return color(red(c), green(c), blue(c), alpha(c) / 2.0);
}

boolean skipDewey(Record r) {
    switch (r.deweyClass) {
        case DEWEY_NA:
            if (!toggle_na) {
                return true;
            }
            break;
        case DEWEY_000:
            if (!toggle_000) {
                return true;
            }
            break;
        case DEWEY_100:
            if (!toggle_100) {
                return true;
            }
            break;
        case DEWEY_200:
            if (!toggle_200) {
                return true;
            }
            break;
        case DEWEY_300:
            if (!toggle_300) {
                return true;
            }
            break;
        case DEWEY_400:
            if (!toggle_400) {
                return true;
            }
            break;
        case DEWEY_500:
            if (!toggle_500) {
                return true;
            }
            break;
        case DEWEY_600:
            if (!toggle_600) {
                return true;
            }
            break;
        case DEWEY_700:
            if (!toggle_700) {
                return true;
            }
            break;
        case DEWEY_800:
            if (!toggle_800) {
                return true;
            }
            break;
        case DEWEY_900:
            if (!toggle_900) {
                return true;
            }
            break;
    }
    return false;
}

boolean skipYear(int i) {
    int year = i + START_YEAR;
    if (year == 2006 && !toggle_2006) return true;
    else if (year == 2007 && !toggle_2007) return true;
    else if (year == 2008 && !toggle_2008) return true;
    else if (year == 2009 && !toggle_2009) return true;
    else if (year == 2010 && !toggle_2010) return true;
    else if (year == 2011 && !toggle_2011) return true;
    else if (year == 2012 && !toggle_2012) return true;
    else if (year == 2013 && !toggle_2013) return true;
    else if (year == 2014 && !toggle_2014) return true;
    else if (year == 2015 && !toggle_2015) return true;
    else if (year == 2016 && !toggle_2016) return true;
    else if (year == 2017 && !toggle_2017) return true;
    else if (year == 2018 && !toggle_2018) return true;
    return false;
}

void setTextAlign(float angle) {
    if (angle >= 0 && angle < PI / 2) {
        textAlign(LEFT, TOP);
    } else if (angle >= PI / 2 && angle < PI) {
        textAlign(RIGHT, TOP);
    } else if (angle >= PI && angle < PI * 4 / 3) {
        textAlign(RIGHT, BOTTOM);
    } else {
        textAlign(LEFT, BOTTOM);
    }
}
