import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.*; 
import java.time.LocalDateTime; 
import java.time.format.DateTimeFormatter; 
import peasy.*; 
import controlP5.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class popular_items extends PApplet {

/********************************************************
 * Data Visualization - Most popular items by year      *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualizing the most checked out items at   *
 *          Seattle Public Library each year from       *
 *          2006 to 2018                                *
 *                                                      *
 * Usage:   Press toggle button on bottom left side     *
 *          to toggle records by year and dewey class.  *
 *          Click on the year list on top right side    *
 *          to display top 50 titles of each year.      *
 *          Use slider to show title of each checkout   *
 *                                                      *
 ********************************************************/







final int START_YEAR = 2006;
final int END_YEAR = 2018;
final int Z_DISTANCE = 11000;
final int ARC_RADIUS = 500;
final int RING_RADIUS = 600;
final int YEAR_SIZE = 100;
final int MONTH_SIZE = 50;
final int TITLE_SIZE = 12;
final int HOVER_SIZE = 80;
final int HOVER_WEIGHT = 80;
final int RING_COLOR = color(200, 127);
final int FONT_COLOR = color(200);
final String[] MONTHS = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
final int INC = 1; // how much each arc increments
final int[] DEWEY_COLOR = {0xffa6cee3,0xff1f78b4,0xffb2df8a,0xff33a02c,0xfffb9a99,0xffe31a1c,0xfffdbf6f,0xffff7f00,0xffcab2d6,0xff6a3d9a,0xffffff99};
final int YEAR_COLOR = color(200);

enum DeweyClass {
    DEWEY_NA, DEWEY_000, DEWEY_100, DEWEY_200, DEWEY_300, DEWEY_400, DEWEY_500, DEWEY_600, DEWEY_700, DEWEY_800, DEWEY_900
}

PeasyCam cam;
PeasyDragHandler singleDragHandler;
ControlP5 cp5;
int sliderValue;
HashMap<Integer, List<Record>> recordMap = new HashMap();
HashMap<String, List<String>> titleMap = new HashMap();
Table table;
int rows;
int years = END_YEAR - START_YEAR + 1;
float theta = (2 * PI) / years;
boolean toggle_na, toggle_000, toggle_100, toggle_200, toggle_300, toggle_400, toggle_500, toggle_600, toggle_700, toggle_800, toggle_900;
boolean toggle_2006, toggle_2007, toggle_2008, toggle_2009, toggle_2010, toggle_2011, toggle_2012, toggle_2013, toggle_2014, toggle_2015, toggle_2016, toggle_2017, toggle_2018;

public void setup() {
    
    textMode(SHAPE);
    loadData();
    setupCamera();
    setupCP5();
}

public void draw() {
    background(0);
    // draw hud
    hud();
    int count = 0;
    int d = 0;
    for (int i = 0; i < years; i++) {
        List<Record> records = recordMap.get(i + START_YEAR);
        // draw ring
        if (skipYear(i)) continue;
        ring(i, d);
        // draw each record
        for (Record r : records) {
            if (skipDewey(r)) continue;
            d += INC;
            pushMatrix();
            translate(0, 0, d);
            strokeWeight(4);
            stroke(r.c);
            float startAngle = map(r.cout.getDayOfYear(), 1, 366, 0, PI * 2);
            float endAngle = map(r.cin.getDayOfYear(), 1, 366, 0, PI * 2);
            // hover text
            if (count == sliderValue) {
                float textAngle = (endAngle - startAngle) / 2 + startAngle;
                strokeWeight(HOVER_WEIGHT);
                setTextAlign(textAngle);
                fill(FONT_COLOR);
                textSize(HOVER_SIZE);
                text(r.title, (RING_RADIUS + HOVER_WEIGHT) * cos(textAngle), (RING_RADIUS + HOVER_WEIGHT) * sin(textAngle));
            }
            noFill();
            arc(0, 0, ARC_RADIUS * 2, ARC_RADIUS * 2, startAngle, endAngle);
            popMatrix();
            count++;
        }
        d += INC;
    }
    setRange(count);
}
public void setupCamera() {
    cam = new PeasyCam(this, Z_DISTANCE);
    cam.setWheelScale(0.1f);
    float fov = PI / 3.0f;
    float cameraZ = (height / 2.0f) / tan(fov / 2.0f);
    perspective(fov, PApplet.parseFloat(width) / PApplet.parseFloat(height), 0.001f, cameraZ * 50.0f);
    singleDragHandler = cam.getRotateDragHandler();
}
public void setupCP5() {
    cp5 = new ControlP5(this);
    // add toggle for dewey class
    addToggle("toggle_na", "N/A", DEWEY_COLOR[0], 15, height-90);
    for (int i = 1; i < 11; i++) {
        String digit = String.format("%03d", (i - 1) * 100);
        addToggle("toggle_" + digit, digit, DEWEY_COLOR[i], 15 + 20 * i, height-90);
    }
    // add toggle for years
    for (int i = 0; i < years; i++) {
        int year = START_YEAR + i;
        addToggle("toggle_" + String.valueOf(year), String.valueOf(year), YEAR_COLOR, 15 + 20 * i, height-60);
    }
    // add year list for selecting year
    List<String> yearList = new ArrayList();
    for (int i = 0; i < years; i++) {
        yearList.add(String.valueOf(i + START_YEAR));
    }
    yearList.add("AllTime");
    cp5.addScrollableList("handleYear")
        .setPosition(width-15-150, 50)
        .setSize(150, height-100)
        .setBarHeight(20)
        .setItemHeight(20)
        .setType(ControlP5.DROPDOWN)
        .setColorForeground(getSemi(YEAR_COLOR))
        .setColorBackground(getSemi(YEAR_COLOR))
        .setColorActive(YEAR_COLOR)
        .setColorLabel(FONT_COLOR)
        .setLabel("YEAR")
        .setOpen(false)
        .onEnter(new CallbackListener() {
            public void controlEvent(CallbackEvent theEvent) {
                cp5.get(ScrollableList.class, "handleTitle").setOpen(false);
            }
        })
        .addItems(yearList);
    // add title list
    cp5.addScrollableList("handleTitle")
        .setPosition(width-15-150, 80)
        .setSize(150, height-100)
        .setBarHeight(20)
        .setItemHeight(20)
        .setType(ControlP5.LIST)
        .setColorForeground(getSemi(YEAR_COLOR))
        .setColorBackground(getSemi(YEAR_COLOR))
        .setColorActive(YEAR_COLOR)
        .setColorLabel(FONT_COLOR)
        .setLabel("TITLE")
        .setOpen(false);
    // add slider for hover text
    cp5.addSlider("sliderValue")
        .setPosition(15,height-30)
        .setSize(400,20)
        .setHandleSize(5)
        .setDecimalPrecision(1)
        .setColorForeground(getSemi(YEAR_COLOR))
        .setColorBackground(getSemi(YEAR_COLOR))
        .setColorActive(YEAR_COLOR)
        .setColorLabel(FONT_COLOR)
        .setLabel("RECORDS")
        .onEnter(new CallbackListener() {
            public void controlEvent(CallbackEvent theEvent) {
                cam.setLeftDragHandler(null);
            }
        })
        .onLeave(new CallbackListener() {
            public void controlEvent(CallbackEvent theEvent) {
                cam.setLeftDragHandler(singleDragHandler);
            }
        });
    cp5.setAutoDraw(false);
}

public void addToggle(String toggle, String dewey, int c, int x, int y) {
    cp5.addToggle(toggle)
        .setPosition(x, y)
        .setColorForeground(getSemi(c))
        .setColorBackground(getSemi(c))
        .setColorActive(c)
        .setColorLabel(FONT_COLOR)
        .setSize(15, 15)
        .setLabel(dewey)
        .setValue(true);
}
public void ring(int i, int d) {
    pushMatrix();
    translate(0, 0, d);
    // draw rings
    strokeWeight(3);
    stroke(RING_COLOR);
    noFill();
    circle(0, 0, RING_RADIUS * 2);
    // draw year text
    fill(FONT_COLOR);
    textSize(YEAR_SIZE);
    float angle = theta * i;
    setTextAlign(angle);
    text(String.valueOf(i + START_YEAR), RING_RADIUS * cos(angle), RING_RADIUS * sin(angle));
    // draw month text
    for (int j = 0; j < 12; j++) {
        textSize(MONTH_SIZE);
        float monthAngle = (2 * PI) / 12 * j;
        setTextAlign(monthAngle);
        text(MONTHS[j], RING_RADIUS * cos(monthAngle), RING_RADIUS * sin(monthAngle));
    }
    popMatrix();
}

public void hud() {
    hint(DISABLE_DEPTH_TEST);
    cam.beginHUD();
    // draw title
    textSize(TITLE_SIZE);
    textAlign(LEFT, TOP);
    fill(FONT_COLOR);
    text("MOST POPULAR ITEMS BY CHECHOUTS 2006-2018\n IN SEATTLE PUBLIC LIBRARY", 15, 15);
    // draw legend

    cp5.draw();
    cam.endHUD();
    hint(ENABLE_DEPTH_TEST);
}
public void handleYear(int i) {
    if (i >= years) {
        cp5.get(ScrollableList.class, "handleTitle")
        .clear()
        .addItems(titleMap.get("AllTime"))
        .setOpen(true);
    } else {
        String year = String.valueOf(i + START_YEAR);
        cp5.get(ScrollableList.class, "handleTitle")
        .clear()
        .addItems(titleMap.get(year))
        .setOpen(true);
    }
}

public void setRange(int max) {
    cp5.get(Slider.class, "sliderValue")
    .setRange(0, max);
}

public void keyPressed() {
    if (key == CODED) {
        if (keyCode == LEFT) {
            cp5.get(Slider.class, "sliderValue")
            .setValue(--sliderValue)
            .update();
        } else if (keyCode == RIGHT) {
            cp5.get(Slider.class, "sliderValue")
            .setValue(++sliderValue)
            .update();
        }
    }
}
public void loadData() {
    // load top checkout titles
    for (int i = START_YEAR; i <= END_YEAR; i++) {
        String filename = "top50_" + String.valueOf(i) + ".csv";
        table = loadTable(filename, "header");
        rows = table.getRowCount();
        List<String> titles = new ArrayList();
        for (int j = 0; j < rows; j++) {
            String title = table.getString(j, 1);
            titles.add(title);
        }
        titleMap.put(String.valueOf(i), titles);
    }
    loadAllTime();
    // load checkout record from csv files
    for (int i = START_YEAR; i <= END_YEAR; i++) {
        String filename = "cout_" + String.valueOf(i) + ".csv";
        table = loadTable(filename, "header");
        rows = table.getRowCount();
        List<Record> records = new ArrayList();
        for (int j = 0; j < rows; j++) {
            String title = table.getString(j, 0);
            int itemNumber = table.getInt(j, 1);
            LocalDateTime cout = convertToDate(table.getString(j, 2));
            LocalDateTime cin = convertToDate(table.getString(j, 3));
            String itemType = table.getString(j, 4);
            String deweyClass = table.getString(j, 5);
            Record record = new Record(title, itemNumber, cout, cin, itemType, deweyClass);
            records.add(record);
        }
        recordMap.put(i, records);
    }
    println("Finished loading records");
}

public void loadAllTime() {
    String filename = "top1000_alltime.csv";
    table = loadTable(filename, "header");
    rows = table.getRowCount();
    List<String> titles = new ArrayList();
    for (int i = 0; i < 50; i++) {
        String title = table.getString(i, 1);
        titles.add(title);
    }
    titleMap.put("AllTime", titles);
}

public LocalDateTime convertToDate(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
    LocalDateTime date = LocalDateTime.parse(str, formatter);
    return date;
}

// return color with semi transparency
public int getSemi(int c) {
    return color(red(c), green(c), blue(c), alpha(c) / 2.0f);
}

public boolean skipDewey(Record r) {
    switch (r.deweyClass) {
        case DEWEY_NA:
            if (!toggle_na) {
                return true;
            }
            break;
        case DEWEY_000:
            if (!toggle_000) {
                return true;
            }
            break;
        case DEWEY_100:
            if (!toggle_100) {
                return true;
            }
            break;
        case DEWEY_200:
            if (!toggle_200) {
                return true;
            }
            break;
        case DEWEY_300:
            if (!toggle_300) {
                return true;
            }
            break;
        case DEWEY_400:
            if (!toggle_400) {
                return true;
            }
            break;
        case DEWEY_500:
            if (!toggle_500) {
                return true;
            }
            break;
        case DEWEY_600:
            if (!toggle_600) {
                return true;
            }
            break;
        case DEWEY_700:
            if (!toggle_700) {
                return true;
            }
            break;
        case DEWEY_800:
            if (!toggle_800) {
                return true;
            }
            break;
        case DEWEY_900:
            if (!toggle_900) {
                return true;
            }
            break;
    }
    return false;
}

public boolean skipYear(int i) {
    int year = i + START_YEAR;
    if (year == 2006 && !toggle_2006) return true;
    else if (year == 2007 && !toggle_2007) return true;
    else if (year == 2008 && !toggle_2008) return true;
    else if (year == 2009 && !toggle_2009) return true;
    else if (year == 2010 && !toggle_2010) return true;
    else if (year == 2011 && !toggle_2011) return true;
    else if (year == 2012 && !toggle_2012) return true;
    else if (year == 2013 && !toggle_2013) return true;
    else if (year == 2014 && !toggle_2014) return true;
    else if (year == 2015 && !toggle_2015) return true;
    else if (year == 2016 && !toggle_2016) return true;
    else if (year == 2017 && !toggle_2017) return true;
    else if (year == 2018 && !toggle_2018) return true;
    return false;
}

public void setTextAlign(float angle) {
    if (angle >= 0 && angle < PI / 2) {
        textAlign(LEFT, TOP);
    } else if (angle >= PI / 2 && angle < PI) {
        textAlign(RIGHT, TOP);
    } else if (angle >= PI && angle < PI * 4 / 3) {
        textAlign(RIGHT, BOTTOM);
    } else {
        textAlign(LEFT, BOTTOM);
    }
}
class Record {
    String title;
    int itemNumber;
    LocalDateTime cout;
    LocalDateTime cin;
    String itemType;
    DeweyClass deweyClass;
    int c;

    Record(String t, int i, LocalDateTime co, LocalDateTime ci, String it, String d){
        this.title = t;
        this.itemNumber = i;
        this.cout = co;
        this.cin = ci;
        this.itemType = it;
        setDeweyClass(d);
        setColor();
    }

    private void setDeweyClass(String dewey) {
        if (dewey.equals("")) {
            this.deweyClass = DeweyClass.DEWEY_NA;
        } else {
            int i = (int)Float.parseFloat(dewey);
            i /= 100;
            if (i == 0) {
                this.deweyClass = DeweyClass.DEWEY_000;
            } else if (i == 1) {
                this.deweyClass = DeweyClass.DEWEY_100;
            } else if (i == 2) {
                this.deweyClass = DeweyClass.DEWEY_200;
            } else if (i == 3) {
                this.deweyClass = DeweyClass.DEWEY_300;
            } else if (i == 4) {
                this.deweyClass = DeweyClass.DEWEY_400;
            } else if (i == 5) {
                this.deweyClass = DeweyClass.DEWEY_500;
            } else if (i == 6) {
                this.deweyClass = DeweyClass.DEWEY_600;
            } else if (i == 7) {
                this.deweyClass = DeweyClass.DEWEY_700;
            } else if (i == 8) {
                this.deweyClass = DeweyClass.DEWEY_800;
            } else if (i == 9) {
                this.deweyClass = DeweyClass.DEWEY_900;
            }
        }
    }

    private void setColor() {
        switch (this.deweyClass) {
            case DEWEY_NA:
                this.c = DEWEY_COLOR[0];
                break;
            case DEWEY_000:
                this.c = DEWEY_COLOR[1];
                break;
            case DEWEY_100:
                this.c = DEWEY_COLOR[2];
                break;
            case DEWEY_200:
                this.c = DEWEY_COLOR[3];
                break;
            case DEWEY_300:
                this.c = DEWEY_COLOR[4];
                break;
            case DEWEY_400:
                this.c = DEWEY_COLOR[5];
                break;
            case DEWEY_500:
                this.c = DEWEY_COLOR[6];
                break;
            case DEWEY_600:
                this.c = DEWEY_COLOR[7];
                break;
            case DEWEY_700:
                this.c = DEWEY_COLOR[8];
                break;
            case DEWEY_800:
                this.c = DEWEY_COLOR[9];
                break;
            case DEWEY_900:
                this.c = DEWEY_COLOR[10];
                break;
        }
    }
}
  public void settings() {  fullScreen(P3D); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "popular_items" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
