/********************************************************
 * Data Visualization - Most popular items by year      *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Visualizing the most checked out items at   *
 *          Seattle Public Library each year from       *
 *          2006 to 2018                                *
 *                                                      *
 * Usage:   Press toggle button on bottom left side     *
 *          to toggle records by year and dewey class.  *
 *          Click on the year list on top right side    *
 *          to display top 50 titles of each year.      *
 *          Use slider to show title of each checkout   *
 *                                                      *
 ********************************************************/

import java.util.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import peasy.*;
import controlP5.*;

final int START_YEAR = 2006;
final int END_YEAR = 2018;
final int Z_DISTANCE = 11000;
final int ARC_RADIUS = 500;
final int RING_RADIUS = 600;
final int YEAR_SIZE = 100;
final int MONTH_SIZE = 50;
final int TITLE_SIZE = 12;
final int HOVER_SIZE = 80;
final int HOVER_WEIGHT = 80;
final color RING_COLOR = color(200, 127);
final color FONT_COLOR = color(200);
final String[] MONTHS = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
final int INC = 1; // how much each arc increments
final color[] DEWEY_COLOR = {#a6cee3,#1f78b4,#b2df8a,#33a02c,#fb9a99,#e31a1c,#fdbf6f,#ff7f00,#cab2d6,#6a3d9a,#ffff99};
final color YEAR_COLOR = color(200);

enum DeweyClass {
    DEWEY_NA, DEWEY_000, DEWEY_100, DEWEY_200, DEWEY_300, DEWEY_400, DEWEY_500, DEWEY_600, DEWEY_700, DEWEY_800, DEWEY_900
}

PeasyCam cam;
PeasyDragHandler singleDragHandler;
ControlP5 cp5;
int sliderValue;
HashMap<Integer, List<Record>> recordMap = new HashMap();
HashMap<String, List<String>> titleMap = new HashMap();
Table table;
int rows;
int years = END_YEAR - START_YEAR + 1;
float theta = (2 * PI) / years;
boolean toggle_na, toggle_000, toggle_100, toggle_200, toggle_300, toggle_400, toggle_500, toggle_600, toggle_700, toggle_800, toggle_900;
boolean toggle_2006, toggle_2007, toggle_2008, toggle_2009, toggle_2010, toggle_2011, toggle_2012, toggle_2013, toggle_2014, toggle_2015, toggle_2016, toggle_2017, toggle_2018;

void setup() {
    fullScreen(P3D);
    textMode(SHAPE);
    loadData();
    setupCamera();
    setupCP5();
}

void draw() {
    background(0);
    // draw hud
    hud();
    int count = 0;
    int d = 0;
    for (int i = 0; i < years; i++) {
        List<Record> records = recordMap.get(i + START_YEAR);
        // draw ring
        if (skipYear(i)) continue;
        ring(i, d);
        // draw each record
        for (Record r : records) {
            if (skipDewey(r)) continue;
            d += INC;
            pushMatrix();
            translate(0, 0, d);
            strokeWeight(4);
            stroke(r.c);
            float startAngle = map(r.cout.getDayOfYear(), 1, 366, 0, PI * 2);
            float endAngle = map(r.cin.getDayOfYear(), 1, 366, 0, PI * 2);
            // hover text
            if (count == sliderValue) {
                float textAngle = (endAngle - startAngle) / 2 + startAngle;
                strokeWeight(HOVER_WEIGHT);
                setTextAlign(textAngle);
                fill(FONT_COLOR);
                textSize(HOVER_SIZE);
                text(r.title, (RING_RADIUS + HOVER_WEIGHT) * cos(textAngle), (RING_RADIUS + HOVER_WEIGHT) * sin(textAngle));
            }
            noFill();
            arc(0, 0, ARC_RADIUS * 2, ARC_RADIUS * 2, startAngle, endAngle);
            popMatrix();
            count++;
        }
        d += INC;
    }
    setRange(count);
}
